@@ -524,8 +524,7 @@ enum ctdb_trans2_commit_error {
 struct ctdb_ltdb_header {
 	uint64_t rsn;
 	uint32_t dmaster;
-	uint16_t laccessor;
-	uint16_t lacount;
+	uint32_t reserved1;
 #define CTDB_REC_FLAG_DEFAULT			0x00000000
 #define CTDB_REC_FLAG_MIGRATED_WITH_DATA	0x00010000
 #define CTDB_REC_FLAG_VACUUM_MIGRATED		0x00020000
@@ -714,10 +713,6 @@ struct ctdb_statistics_wire {
 /*
  * db statistics
  */
-struct ctdb_db_hot_key {
-	uint32_t count;
-	TDB_DATA key;
-};
 struct ctdb_db_statistics {
 	struct {
 		uint32_t num_calls;
@@ -731,14 +726,11 @@ struct ctdb_db_statistics {
 	uint32_t db_ro_revokes;
 	uint32_t hop_count_bucket[MAX_COUNT_BUCKETS];
 	uint32_t num_hot_keys;
-	struct ctdb_db_hot_key hot_keys[MAX_HOT_KEYS];
-};
-struct ctdb_db_statistics_wire {
-	uint32_t db_ro_delegations;
-	uint32_t db_ro_revokes;
-	uint32_t hop_count_bucket[MAX_COUNT_BUCKETS];
-	uint32_t num_hot_keys;
-	char hot_keys[1];
+	struct {
+		uint32_t count;
+		TDB_DATA key;
+	} hot_keys[MAX_HOT_KEYS];
+	char hot_keys_wire[1];
 };
 
 /*
