@@ -21,7 +21,7 @@
   protocol design and packet details
 */
 #include "includes.h"
-#include "lib/tdb/include/tdb.h"
+#include "tdb.h"
 #include "lib/util/dlinklist.h"
 #include "system/network.h"
 #include "system/filesys.h"
@@ -131,12 +131,12 @@ static void ctdb_call_send_redirect(struct ctdb_context *ctdb,
 	}
 	c->hopcount++;
 
-	if (c->hopcount%100 == 99) {
-		DEBUG(DEBUG_WARNING,("High hopcount %d dbid:0x%08x "
-			"key:0x%08x pnn:%d src:%d lmaster:%d "
+	if (c->hopcount%100 > 95) {
+		DEBUG(DEBUG_WARNING,("High hopcount %d dbid:%s "
+			"key:0x%08x reqid=%08x pnn:%d src:%d lmaster:%d "
 			"header->dmaster:%d dst:%d\n",
-			c->hopcount, ctdb_db->db_id, ctdb_hash(&key),
-			ctdb->pnn, c->hdr.srcnode, lmaster,
+			c->hopcount, ctdb_db->db_name, ctdb_hash(&key),
+			c->hdr.reqid, ctdb->pnn, c->hdr.srcnode, lmaster,
 			header->dmaster, c->hdr.destnode));
 	}
 
@@ -399,7 +399,7 @@ static void ctdb_become_dmaster(struct ctdb_db_context *ctdb_db,
 		return;
 	}
 
-	ctdb_call_local(ctdb_db, state->call, &header, state, &data, true, ctdb->pnn);
+	ctdb_call_local(ctdb_db, state->call, &header, state, &data, true);
 
 	ret = ctdb_ltdb_unlock(ctdb_db, state->call->key);
 	if (ret != 0) {
@@ -564,7 +564,9 @@ ctdb_make_record_sticky(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb_
 	sr->ctdb_db = ctdb_db;
 	sr->pindown = NULL;
 
-	DEBUG(DEBUG_ERR,("Make record sticky in db %s\n", ctdb_db->db_name));
+	DEBUG(DEBUG_ERR,("Make record sticky for %d seconds in db %s key:0x%08x.\n",
+			 ctdb->tunable.sticky_duration,
+			 ctdb_db->db_name, ctdb_hash(&key)));
 
 	trbt_insertarray32_callback(ctdb_db->sticky_records, k[0], &k[0], ctdb_make_sticky_record_callback, sr);
 
@@ -658,7 +660,7 @@ ctdb_defer_pinned_down_request(struct ctdb_context *ctdb, struct ctdb_db_context
 static void
 ctdb_update_db_stat_hot_keys(struct ctdb_db_context *ctdb_db, TDB_DATA key, int hopcount)
 {
-	int i;
+	int i, id;
 
 	/* smallest value is always at index 0 */
 	if (hopcount <= ctdb_db->statistics.hot_keys[0].count) {
@@ -681,16 +683,27 @@ ctdb_update_db_stat_hot_keys(struct ctdb_db_context *ctdb_db, TDB_DATA key, int
 		goto sort_keys;
 	}
 
-	if (ctdb_db->statistics.hot_keys[0].key.dptr != NULL) {
-		talloc_free(ctdb_db->statistics.hot_keys[0].key.dptr);
+	if (ctdb_db->statistics.num_hot_keys < MAX_HOT_KEYS) {
+		id = ctdb_db->statistics.num_hot_keys;
+		ctdb_db->statistics.num_hot_keys++;
+	} else {
+		id = 0;
 	}
-	ctdb_db->statistics.hot_keys[0].key.dsize = key.dsize;
-	ctdb_db->statistics.hot_keys[0].key.dptr  = talloc_memdup(ctdb_db, key.dptr, key.dsize);
-	ctdb_db->statistics.hot_keys[0].count = hopcount;
 
+	if (ctdb_db->statistics.hot_keys[id].key.dptr != NULL) {
+		talloc_free(ctdb_db->statistics.hot_keys[id].key.dptr);
+	}
+	ctdb_db->statistics.hot_keys[id].key.dsize = key.dsize;
+	ctdb_db->statistics.hot_keys[id].key.dptr  = talloc_memdup(ctdb_db, key.dptr, key.dsize);
+	ctdb_db->statistics.hot_keys[id].count = hopcount;
+	DEBUG(DEBUG_NOTICE,("Updated hot key database=%s key=0x%08x id=%d hop_count=%d\n",
+			    ctdb_db->db_name, ctdb_hash(&key), id, hopcount));
 
 sort_keys:
-	for (i = 2; i < MAX_HOT_KEYS; i++) {
+	for (i = 1; i < MAX_HOT_KEYS; i++) {
+		if (ctdb_db->statistics.hot_keys[i].count == 0) {
+			continue;
+		}
 		if (ctdb_db->statistics.hot_keys[i].count < ctdb_db->statistics.hot_keys[0].count) {
 			hopcount = ctdb_db->statistics.hot_keys[i].count;
 			ctdb_db->statistics.hot_keys[i].count = ctdb_db->statistics.hot_keys[0].count;
@@ -748,7 +761,9 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 	*/
 	if (ctdb_db->sticky) {
 		if (ctdb_defer_pinned_down_request(ctdb, ctdb_db, call->key, hdr) == 0) {
-		  DEBUG(DEBUG_WARNING,("Defer request for pinned down record in %s\n", ctdb_db->db_name));
+			DEBUG(DEBUG_WARNING,
+			      ("Defer request for pinned down record in %s\n", ctdb_db->db_name));
+			talloc_free(call);
 			return;
 		}
 	}
@@ -762,10 +777,12 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 					   ctdb_call_input_pkt, ctdb, false);
 	if (ret == -1) {
 		ctdb_send_error(ctdb, hdr, ret, "ltdb fetch failed in ctdb_request_call");
+		talloc_free(call);
 		return;
 	}
 	if (ret == -2) {
 		DEBUG(DEBUG_INFO,(__location__ " deferred ctdb_request_call\n"));
+		talloc_free(call);
 		return;
 	}
 
@@ -815,6 +832,7 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 		if (ret != 0) {
 			DEBUG(DEBUG_ERR,(__location__ " ctdb_ltdb_unlock() failed with error %d\n", ret));
 		}
+		talloc_free(call);
 		return;
 	}
 
@@ -891,6 +909,7 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 		CTDB_INCREMENT_DB_STAT(ctdb_db, db_ro_delegations);
 
 		talloc_free(r);
+		talloc_free(call);
 		return;
 	}
 
@@ -913,20 +932,15 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 	   should make it sticky.
 	*/
 	if (ctdb_db->sticky && c->hopcount >= ctdb->tunable.hopcount_make_sticky) {
-		DEBUG(DEBUG_ERR, ("Hot record in database %s. Hopcount is %d. Make record sticky for %d seconds\n", ctdb_db->db_name, c->hopcount, ctdb->tunable.sticky_duration));
 		ctdb_make_record_sticky(ctdb, ctdb_db, call->key);
 	}
 
 
-	/* if this nodes has done enough consecutive calls on the same record
-	   then give them the record
-	   or if the node requested an immediate migration
-	*/
-	if ( c->hdr.srcnode != ctdb->pnn &&
-	     ((header.laccessor == c->hdr.srcnode
-	       && header.lacount >= ctdb->tunable.max_lacount
-	       && ctdb->tunable.max_lacount != 0)
-	      || (c->flags & CTDB_IMMEDIATE_MIGRATION)) ) {
+	/* Try if possible to migrate the record off to the caller node.
+	 * From the clients perspective a fetch of the data is just as 
+	 * expensive as a migration.
+	 */
+	if (c->hdr.srcnode != ctdb->pnn) {
 		if (ctdb_db->transaction_active) {
 			DEBUG(DEBUG_INFO, (__location__ " refusing migration"
 			      " of key %s while transaction is active\n",
@@ -941,11 +955,12 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 			if (ret != 0) {
 				DEBUG(DEBUG_ERR,(__location__ " ctdb_ltdb_unlock() failed with error %d\n", ret));
 			}
-			return;
 		}
+		talloc_free(call);
+		return;
 	}
 
-	ret = ctdb_call_local(ctdb_db, call, &header, hdr, &data, true, c->hdr.srcnode);
+	ret = ctdb_call_local(ctdb_db, call, &header, hdr, &data, true);
 	if (ret != 0) {
 		DEBUG(DEBUG_ERR,(__location__ " ctdb_call_local failed\n"));
 		call->status = -1;
@@ -971,14 +986,15 @@ void ctdb_request_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 	ctdb_queue_packet(ctdb, &r->hdr);
 
 	talloc_free(r);
+	talloc_free(call);
 }
 
-/*
-  called when a CTDB_REPLY_CALL packet comes in
-
-  This packet comes in response to a CTDB_REQ_CALL request packet. It
-  contains any reply data from the call
-*/
+/**
+ * called when a CTDB_REPLY_CALL packet comes in
+ *
+ * This packet comes in response to a CTDB_REQ_CALL request packet. It
+ * contains any reply data from the call
+ */
 void ctdb_reply_call(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 {
 	struct ctdb_reply_call *c = (struct ctdb_reply_call *)hdr;
@@ -1072,9 +1088,9 @@ finished_ro:
 /*
   called when a CTDB_REPLY_DMASTER packet comes in
 
-  This packet comes in from the lmaster response to a CTDB_REQ_CALL
+  This packet comes in from the lmaster in response to a CTDB_REQ_CALL
   request packet. It means that the current dmaster wants to give us
-  the dmaster role
+  the dmaster role.
 */
 void ctdb_reply_dmaster(struct ctdb_context *ctdb, struct ctdb_req_header *hdr)
 {
@@ -1232,7 +1248,7 @@ struct ctdb_call_state *ctdb_call_local_send(struct ctdb_db_context *ctdb_db,
 	*(state->call) = *call;
 	state->ctdb_db = ctdb_db;
 
-	ret = ctdb_call_local(ctdb_db, state->call, header, state, data, true, ctdb->pnn);
+	ret = ctdb_call_local(ctdb_db, state->call, header, state, data, true);
 	if (ret != 0) {
 		DEBUG(DEBUG_DEBUG,("ctdb_call_local() failed, ignoring return code %d\n", ret));
 	}
@@ -1645,6 +1661,7 @@ int ctdb_start_revoke_ro_record(struct ctdb_context *ctdb, struct ctdb_db_contex
 		close(rc->fd[0]);
 		debug_extra = talloc_asprintf(NULL, "revokechild-%s:", ctdb_db->db_name);
 
+		ctdb_set_process_name("ctdb_revokechild");
 		if (switch_from_server_to_client(ctdb, "revokechild-%s", ctdb_db->db_name) != 0) {
 			DEBUG(DEBUG_ERR,("Failed to switch from server to client for revokechild process\n"));
 			c = 1;
