@@ -18,7 +18,7 @@
 */
 
 #include "includes.h"
-#include "lib/tdb/include/tdb.h"
+#include "tdb.h"
 #include "system/network.h"
 #include "system/filesys.h"
 #include "system/dir.h"
@@ -706,7 +706,7 @@ int ctdb_set_db_readonly(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb
 	}
 
 	if (ctdb_db->persistent) {
-		DEBUG(DEBUG_ERR,("Trying to set persistent database with readonly property\n"));
+		DEBUG(DEBUG_ERR,("Persistent databases do not support readonly property\n"));
 		return -1;
 	}
 
@@ -728,6 +728,9 @@ int ctdb_set_db_readonly(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb
 	DEBUG(DEBUG_NOTICE,("OPENED tracking database : '%s'\n", ropath));
 
 	ctdb_db->readonly = true;
+
+	DEBUG(DEBUG_NOTICE, ("Readonly property set on DB %s\n", ctdb_db->db_name));
+
 	talloc_free(ropath);
 	return 0;
 }
@@ -993,8 +996,9 @@ again:
 	}
 
 
-	DEBUG(DEBUG_INFO,("Attached to database '%s'\n", ctdb_db->db_path));
-	
+	DEBUG(DEBUG_NOTICE,("Attached to database '%s' with flags 0x%x\n",
+			    ctdb_db->db_path, tdb_flags));
+
 	/* success */
 	return 0;
 }
@@ -1183,7 +1187,10 @@ static int ctdb_attach_persistent(struct ctdb_context *ctdb,
 		int invalid_name = 0;
 		
 		s = talloc_strdup(ctdb, de->d_name);
-		CTDB_NO_MEMORY(ctdb, s);
+		if (s == NULL) {
+			closedir(d);
+			CTDB_NO_MEMORY(ctdb, s);
+		}
 
 		/* only accept names ending in .tdb */
 		p = strstr(s, ".tdb.");
@@ -1477,9 +1484,6 @@ int32_t ctdb_control_set_db_priority(struct ctdb_context *ctdb, TDB_DATA indata)
 
 int ctdb_set_db_sticky(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb_db)
 {
-
-	DEBUG(DEBUG_NOTICE,("set db sticky %s\n", ctdb_db->db_name));
-
 	if (ctdb_db->sticky) {
 		return 0;
 	}
@@ -1493,6 +1497,8 @@ int ctdb_set_db_sticky(struct ctdb_context *ctdb, struct ctdb_db_context *ctdb_d
 
 	ctdb_db->sticky = true;
 
+	DEBUG(DEBUG_NOTICE,("set db sticky %s\n", ctdb_db->db_name));
+
 	return 0;
 }
 
@@ -1501,7 +1507,7 @@ int32_t ctdb_control_get_db_statistics(struct ctdb_context *ctdb,
 				TDB_DATA *outdata)
 {
 	struct ctdb_db_context *ctdb_db;
-	struct ctdb_db_statistics_wire *stats;
+	struct ctdb_db_statistics *stats;
 	int i;
 	int len;
 	char *ptr;
@@ -1512,33 +1518,25 @@ int32_t ctdb_control_get_db_statistics(struct ctdb_context *ctdb,
 		return -1;
 	}
 
-	len = offsetof(struct ctdb_db_statistics_wire, hot_keys);
+	len = offsetof(struct ctdb_db_statistics, hot_keys_wire);
 	for (i = 0; i < MAX_HOT_KEYS; i++) {
-		len += 8 + ctdb_db->statistics.hot_keys[i].key.dsize;
+		len += ctdb_db->statistics.hot_keys[i].key.dsize;
 	}
 
 	stats = talloc_size(outdata, len);
 	if (stats == NULL) {
-		DEBUG(DEBUG_ERR,("Failed to allocate db statistics wire structure\n"));
+		DEBUG(DEBUG_ERR,("Failed to allocate db statistics structure\n"));
 		return -1;
 	}
 
-	stats->db_ro_delegations = ctdb_db->statistics.db_ro_delegations;
-	stats->db_ro_revokes     = ctdb_db->statistics.db_ro_revokes;
-	for (i = 0; i < MAX_COUNT_BUCKETS; i++) {
-		stats->hop_count_bucket[i] = ctdb_db->statistics.hop_count_bucket[i];
-	}
+	*stats = ctdb_db->statistics;
+
 	stats->num_hot_keys = MAX_HOT_KEYS;
 
-	ptr = &stats->hot_keys[0];
+	ptr = &stats->hot_keys_wire[0];
 	for (i = 0; i < MAX_HOT_KEYS; i++) {
-		*(uint32_t *)ptr = ctdb_db->statistics.hot_keys[i].count;
-		ptr += 4;
-
-		*(uint32_t *)ptr = ctdb_db->statistics.hot_keys[i].key.dsize;
-		ptr += 4;
-
-		memcpy(ptr, ctdb_db->statistics.hot_keys[i].key.dptr, ctdb_db->statistics.hot_keys[i].key.dsize);
+		memcpy(ptr, ctdb_db->statistics.hot_keys[i].key.dptr,
+		       ctdb_db->statistics.hot_keys[i].key.dsize);
 		ptr += ctdb_db->statistics.hot_keys[i].key.dsize;
 	}
 
