@@ -115,6 +115,17 @@ setup_generic ()
     rm -f "$FAKE_IP_STATE"/*/*
     rm -f "$FAKE_IP_STATE"/* 2>/dev/null || true
     rmdir "$FAKE_IP_STATE"/* 2>/dev/null || true
+
+
+    export CTDB_DBDIR="${EVENTSCRIPTS_TESTS_VAR_DIR}/db"
+    mkdir -p "${CTDB_DBDIR}/persistent"
+
+    export FAKE_TDBTOOL_SUPPORTS_CHECK="yes"
+    export FAKE_TDB_IS_OK
+    export FAKE_DATE_OUTPUT
+
+    export FAKE_NETSTAT_TCP_ESTABLISHED FAKE_TCP_LISTEN FAKE_NETSTAT_UNIX_LISTEN
+    export FAKE_NETSTAT_TCP_ESTABLISHED_FILE=$(mktemp --tmpdir="$EVENTSCRIPTS_TESTS_VAR_DIR")
 }
 
 tcp_port_down ()
@@ -275,9 +286,10 @@ setup_ctdb ()
     export FAKE_CTDB_PNN="${2:-0}"
     echo "Setting up CTDB with PNN ${FAKE_CTDB_PNN}"
 
+    export CTDB_PUBLIC_ADDRESSES="${CTDB_BASE}/public_addresses"
     if [ -n "$3" ] ; then
 	echo "Setting up CTDB_PUBLIC_ADDRESSES: $3"
-	export CTDB_PUBLIC_ADDRESSES=$(mktemp)
+	CTDB_PUBLIC_ADDRESSES=$(mktemp)
 	for _i in $3 ; do
 	    _ip="${_i%@*}"
 	    _ifaces="${_i#*@}"
@@ -295,6 +307,8 @@ setup_ctdb ()
     export FAKE_CTDB_SCRIPTSTATUS="$FAKE_CTDB_STATE/scriptstatus"
     mkdir -p "$FAKE_CTDB_SCRIPTSTATUS"
     rm -f "$FAKE_CTDB_SCRIPTSTATUS"/*
+
+    export CTDB_PARTIALLY_ONLINE_INTERFACES
 }
 
 ctdb_get_interfaces ()
@@ -555,6 +569,9 @@ setup_nfs ()
 
     export CTDB_NFS_SKIP_SHARE_CHECK="no"
 
+    export CTDB_MONITOR_NFS_THREAD_COUNT RPCNFSDCOUNT FAKE_NFSD_THREAD_PIDS
+    export CTDB_NFS_DUMP_STUCK_THREADS
+
     # Reset the failcounts for nfs services.
     eventscript_call eval rm -f '$ctdb_fail_dir/nfs_*'
 
@@ -579,6 +596,20 @@ setup_nfs ()
     fi
 }
 
+setup_nfs_ganesha ()
+{
+    setup_nfs "$@"
+    export NFS_SERVER_MODE="ganesha"
+    if [ "$1" != "down" ] ; then
+	export CTDB_MANAGES_NFS="yes"
+    fi
+
+    # We do not support testing the Ganesha-nfsd-specific part of the
+    # eventscript.
+    export CTDB_SKIP_GANESHA_NFSD_CHECK="yes"
+    export CTDB_NFS_SKIP_SHARE_CHECK="yes"
+}
+
 rpc_services_down ()
 {
     for _i ; do
@@ -644,7 +675,17 @@ rpc_set_service_failure_response ()
 	    \#*) continue ;;
 	esac
 
-	if [ "$_numfails" "$_op" "$_li" ] ; then
+	_hit=false
+	if [ "$_op" != "%" ] ; then
+	    if [ $_numfails $_op $_li ] ; then
+		_hit=true
+	    fi
+	else
+	    if [ $(($_numfails $_op $_li)) -eq 0 ] ; then
+		_hit=true
+	    fi
+	fi
+	if $_hit ; then
 	    _out=""
 	    _rc=0
 	    for _action in $_actions ; do
@@ -662,35 +703,38 @@ ERROR: $_pn failed RPC check:
 rpcinfo: RPC: Program not registered
 program $_pn version $_ver is not available"
 			;;
-		    restart|restart:*)
-			_opts=""
+		    restart*)
 			_p="rpc.${_progname}"
-			case "$_progname" in
-			    statd)
-				_opts="${STATD_HOSTNAME:+ -n }${STATD_HOSTNAME}"
-				;;
+			case "$_action" in
+			    *:b) _bg="&" ;;
+			    *)   _bg=""  ;;
 			esac
-			case "${_progname}${_action#restart}" in
+			case "$_progname" in
 			    nfsd)
 				_t="\
-Trying to restart NFS service
-Starting nfslock: OK
-Starting nfs: OK"
+Trying to restart NFS service"
+
+				if [ -n "$CTDB_NFS_DUMP_STUCK_THREADS" ] ; then
+				    for _pid in $FAKE_NFSD_THREAD_PIDS ; do
+					_t="\
+$_t
+${_bg}Stack trace for stuck nfsd thread [${_pid}]:
+${_bg}[<ffffffff87654321>] fake_stack_trace_for_pid_${_pid}/stack+0x0/0xff"
+				    done
+				fi
+
+				_t="\
+${_t}
+${_bg}Starting nfslock: OK
+${_bg}Starting nfs: OK"
 				;;
-			    nfsd:bs)
-				_t="Trying to restart NFS service"
-				;;
-			    lockd|lockd:b)
+			    lockd)
 				_t="\
 Trying to restart lock manager service
-Stopping nfslock: OK
-Starting nfslock: OK"
-				;;
-			    lockd:*)
-				_t="Trying to restart lock manager service"
+${_bg}Starting nfslock: OK"
 				;;
 			    *)
-				_t="Trying to restart $_progname [${_p}${_opts}]"
+				_t="Trying to restart $_progname [${_p}]"
 			esac
 			_out="${_out}${_out:+${_nl}}${_t}"
 			;;
@@ -790,11 +834,6 @@ EOF
 
 # Any args are passed to the eventscript.
 
-# Eventscript tracing can be done by setting:
-#   EVENTSCRIPTS_TESTS_TRACE="sh -x"
-
-# or similar.  This will almost certainly make a test fail but is
-# useful for debugging.
 simple_test ()
 {
     [ -n "$event" ] || die 'simple_test: $event not set'
@@ -899,7 +938,11 @@ iterate_test ()
 	    shift 2
 	fi
 
-	_out=$($EVENTSCRIPTS_TESTS_TRACE "${CTDB_BASE}/events.d/$script" "$event" $args 2>&1)
+	_trace=""
+	if $TEST_COMMAND_TRACE ; then
+	    _trace="sh -x"
+	fi
+	_out=$($_trace "${CTDB_BASE}/events.d/$script" "$event" $args 2>&1)
 	_rc=$?
 
     if [ -n "$OUT_FILTER" ] ; then
